@@ -5,16 +5,14 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
-#include <curl/curl.h>
 #include <stdio.h>
 #include "fmt.h"
-#include "md5.h"
+#include "plugin.h"
 #include "scrobbler.h"
 #include "config.h"
 #include <glib.h>
-
-#include <audacious/titlestring.h>
-#include <audacious/util.h>
+#include <audacious/plugin.h>
+#include <libaudcore/md5.h>
 
 #define SCROBBLER_HS_URL "http://post.gerpok.com"
 #define SCROBBLER_CLI_ID "aud"
@@ -24,6 +22,7 @@
 #define SCROBBLER_IMPLEMENTATION "0.1"		/* This is the implementation, not the player version. */
 #define SCROBBLER_SB_MAXLEN 1024
 #define CACHE_SIZE 1024
+#define ALLOW_MULTIPLE
 
 /* Scrobblerbackend for xmms plugin, first draft */
 
@@ -85,16 +84,17 @@ static void q_item_free(item_t *item)
 	free(item);
 }
 
-static void q_put(TitleInput *tuple, int len)
+static void q_put(Tuple *tuple, int len)
 {
 	item_t *item;
+	const gchar *album;
 
 	item = malloc(sizeof(item_t));
 
-	item->artist = fmt_escape(tuple->performer);
-	item->title = fmt_escape(tuple->track_name);
+	item->artist = fmt_escape(aud_tuple_get_string(tuple, FIELD_ARTIST, NULL));
+	item->title = fmt_escape(aud_tuple_get_string(tuple, FIELD_TITLE, NULL));
 	item->utctime = fmt_escape(fmt_timestr(time(NULL), 1));
-	snprintf(item->len, sizeof(item->len), "%d", len);
+	g_snprintf(item->len, sizeof(item->len), "%d", len);
 
 #ifdef NOTYET
 	if(tuple->mb == NULL)
@@ -105,10 +105,10 @@ static void q_put(TitleInput *tuple, int len)
 		item->mb = fmt_escape((char*)tuple->mb);
 #endif
 
-	if(tuple->album_name == NULL)
+	if((album = aud_tuple_get_string(tuple, FIELD_ALBUM, NULL)))
 		item->album = fmt_escape("");
 	else
-		item->album = fmt_escape((char*)tuple->album_name);
+		item->album = fmt_escape((char*) album);
 
 	q_nitems++;
 
@@ -411,11 +411,12 @@ static int gerpok_sc_handshake(void)
 	char buf[4096];
 	CURL *curl;
 
-	snprintf(buf, sizeof(buf), "%s/?hs=true&p=%s&c=%s&v=%s&u=%s",
+	g_snprintf(buf, sizeof(buf), "%s/?hs=true&p=%s&c=%s&v=%s&u=%s",
 			SCROBBLER_HS_URL, SCROBBLER_VERSION,
 			SCROBBLER_CLI_ID, SCROBBLER_IMPLEMENTATION, gerpok_sc_username);
 
 	curl = curl_easy_init();
+        setup_proxy(curl);
 	curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1);
 	curl_easy_setopt(curl, CURLOPT_URL, buf);
 	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, 
@@ -444,17 +445,17 @@ static int gerpok_sc_handshake(void)
 	}
 
 	if (gerpok_sc_challenge_hash != NULL) {
-		md5_state_t md5state;
+		aud_md5state_t md5state;
 		unsigned char md5pword[16];
 		
-		md5_init(&md5state);
+		aud_md5_init(&md5state);
 		/*pdebug(fmt_vastr("Pass Hash: %s", gerpok_sc_password), DEBUG);*/
-		md5_append(&md5state, (unsigned const char *)gerpok_sc_password,
+		aud_md5_append(&md5state, (unsigned const char *)gerpok_sc_password,
 				strlen(gerpok_sc_password));
 		/*pdebug(fmt_vastr("Challenge Hash: %s", gerpok_sc_challenge_hash), DEBUG);*/
-		md5_append(&md5state, (unsigned const char *)gerpok_sc_challenge_hash,
+		aud_md5_append(&md5state, (unsigned const char *)gerpok_sc_challenge_hash,
 				strlen(gerpok_sc_challenge_hash));
-		md5_finish(&md5state, md5pword);
+		aud_md5_finish(&md5state, md5pword);
 		hexify((char*)md5pword, sizeof(md5pword));
 		/*pdebug(fmt_vastr("Response Hash: %s", gerpok_sc_response_hash), DEBUG);*/
 	}
@@ -567,9 +568,9 @@ static int gerpok_sc_parse_sb_res(void)
 
 static gchar *gerpok_sc_itemtag(char c, int n, char *str)
 {
-    static char buf[SCROBBLER_SB_MAXLEN]; 
-    snprintf(buf, SCROBBLER_SB_MAXLEN, "&%c[%d]=%s", c, n, str);
-    return buf;
+	static char buf[SCROBBLER_SB_MAXLEN]; 
+	g_snprintf(buf, SCROBBLER_SB_MAXLEN, "&%c[%d]=%s", c, n, str);
+	return buf;
 }
 
 #define cfa(f, l, n, v) \
@@ -621,6 +622,7 @@ static int gerpok_sc_submitentry(gchar *entry)
         GString *submission;
 
 	curl = curl_easy_init();
+        setup_proxy(curl);
 	curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1);
 	curl_easy_setopt(curl, CURLOPT_URL, gerpok_sc_submit_url);
 	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION,
@@ -744,10 +746,13 @@ static void read_cache(void)
 	char buf[PATH_MAX], *cache = NULL, *ptr1, *ptr2;
 	int cachesize, written, i = 0;
 	item_t *item;
+	gchar* config_datadir;
 
 	cachesize = written = 0;
 
-	snprintf(buf, sizeof(buf), "%s/gerpokqueue.txt", audacious_get_localdir());
+	config_datadir = audacious_get_localdir();
+	g_snprintf(buf, sizeof(buf), "%s/gerpokqueue.txt", config_datadir);
+	g_free(config_datadir);
 
 	if (!(fd = fopen(buf, "r")))
 		return;
@@ -822,6 +827,7 @@ static void dump_queue(void)
 	FILE *fd;
 	item_t *item;
 	char *home, buf[PATH_MAX];
+	gchar* config_datadir;
 
 	/*pdebug("Entering dump_queue();", DEBUG);*/
 
@@ -831,7 +837,9 @@ static void dump_queue(void)
 		return;
 	}
 
-	snprintf(buf, sizeof(buf), "%s/gerpokqueue.txt", audacious_get_localdir());
+	config_datadir = audacious_get_localdir();
+	g_snprintf(buf, sizeof(buf), "%s/gerpokqueue.txt", config_datadir);
+	g_free(config_datadir);
 
 	if (!(fd = fopen(buf, "w")))
 	{
@@ -926,7 +934,7 @@ void gerpok_sc_init(char *uname, char *pwd)
 	pdebug("scrobbler starting up", DEBUG);
 }
 
-void gerpok_sc_addentry(GMutex *mutex, TitleInput *tuple, int len)
+void gerpok_sc_addentry(GMutex *mutex, Tuple *tuple, int len)
 {
 	g_mutex_lock(mutex);
 	q_put(tuple, len);
